# Generated by Django 3.0.12 on 2021-03-10 10:17

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("pretixbase", "0175_orderrefund_comment"),
    ]

    operations = [
        migrations.CreateModel(
            name="ItemConfig",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ("days", models.IntegerField()),
                (
                    "event",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="pretixbase.Event",
                    ),
                ),
                (
                    "item",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="vacc_sautosched_config",
                        to="pretixbase.Item",
                    ),
                ),
            ],
        ),
    ]
