{% extends "pretixpresale/event/base.html" %}
{% load bootstrap3 %}
{% load i18n %}
{% load rich_text %}

{% block content %}

<p>
{{ request.event.settings.vacc_autosched_self_service_info | rich_text }}
</p>

<form method="post">
    {% csrf_token %}
    {% bootstrap_form_errors form %}
    {% bootstrap_form form %}
    <button type="submit" class="btn btn-primary btn-block">{% trans "Book second dose" %}</button>
</form>

{% endblock %}
